#include "DomRGBScreen.h"
#include <stdlib.h>
#include <arduino.h>
#include "HardwareSerial.h"
#include "utils.h"

#define pinTest 10

DomRGBScreen::DomRGBScreen(int i) {
  
}

void DomRGBScreen::begin() {
    // calculate how much time we have to show the brightest plane for 1 row
  float minPlaneOEFreq = 1.0 / ((1.0/(float)(SCR_FREQ * MAX_PLANES * MAX_ROWS)) - PLANE_ROW_OVERHEAD);

  Serial.print("Max freq: ");
  Serial.print(minPlaneOEFreq);
  Serial.println();
 
  float f = minPlaneOEFreq;
  int ticks_main = (int)(F_CPU / 2 / (SCR_FREQ * MAX_PLANES * MAX_ROWS));
  int ticks = (int)(F_CPU / 2 / f);
  //int ticks = 4096;
  for (int i = 0; i < MAX_PLANES; i++) {
      plane_ticks[i] = (ticks_main - (ticks + DEAD_TICKS));

      Serial.println(plane_ticks[i]);      
      ticks = ticks >> 1;
  }


  Serial.print("TICKS MAIN: ");
  Serial.println(ticks_main);

  pinMode(pinADD_A, OUTPUT);
  pinMode(pinADD_B, OUTPUT);
  pinMode(pinADD_C, OUTPUT);
  pinMode(pinADD_D, OUTPUT);
  pinMode(pinLAT, OUTPUT);
  pinMode(pinCLK, OUTPUT);

  pinMode(pinR, OUTPUT);
  pinMode(pinG, OUTPUT);
  pinMode(pinB, OUTPUT);
  
  pinMode(pinR1, OUTPUT);
  pinMode(pinG1, OUTPUT);
  pinMode(pinB1, OUTPUT);
  
  pinMode(pinR2, OUTPUT);
  pinMode(pinG2, OUTPUT);
  pinMode(pinB2, OUTPUT);
  
  pinMode(pinR3, OUTPUT);
  pinMode(pinG3, OUTPUT);
  pinMode(pinB3, OUTPUT);

  digitalWrite(pinLAT, 0);
  digitalWrite(pinCLK, 0);

  Serial.println("here");

  flip(0);

  memset(scr_a, 128, SCR_WIDTH * SCR_HEIGHT * 3);
  memset(scr_b, 64, SCR_WIDTH * SCR_HEIGHT * 3);
  
  //WRITE ENABLE
  FTM0_MODE |= FTM_MODE_WPDIS; // | FTM_MODE_FTMEN;
  //Disable everything before setting mod
  FTM0_SC = 0;
  //Timer modulus
  FTM0_MOD = ticks_main;
  //System clock, prescaler 0, enable modulus overflow interrupt
  FTM0_SC = FTM_SC_CLKS(1) | FTM_SC_PS(0) | FTM_SC_TOIE;
  //Set pin 6 as timer output with high current and slew rate
  PORTD_PCR4 = PORT_PCR_MUX(4) | PORT_PCR_DSE | PORT_PCR_SRE;
  //reset counter
  FTM0_CNT = 0;
  //reset counter start
  FTM0_CNTIN = 0;
  //set channel options for PWM inverted
  FTM0_C4SC = FTM_CSC_MSB | FTM_CSC_ELSB;
  //initial pwm value
  FTM0_C4V = 3;
   // Enable FTM0 interrupt inside NVIC
  NVIC_ENABLE_IRQ(IRQ_FTM0);

}

DomRGBScreen::~DomRGBScreen() {
  free(scr_a);
  free(scr_b);
}


void DomRGBScreen::flip(int i) {
  unsigned char *x;
  unsigned char *y;
  if (i & 1) {
    x = scr_a; 
    y = scr_b;
  } else {
    x = scr_b;
    y = scr_a;
  }

  scr_base = x;

  vid_base = y;

}



volatile unsigned char scan_plane = 0;
volatile unsigned char scan_row = 0;
unsigned int scan_mask_R = 0;
unsigned int scan_mask_G = 0;
unsigned int scan_mask_B = 0;

unsigned int prev_scan_row;
unsigned int prev_scan_plane;

volatile int xxx = 0;

#define DWR(PIN, MASK) if (x & MASK) z |= PIN; 

extern "C" void ftm0_isr(void) {
  if (FTM0_SC & FTM_SC_TOF)
  {
    FTM0_SC &= ~FTM_SC_TOF;
    _domRGBScreen.updateDisplay();
  }
}


void DomRGBScreen::updateDisplay() 
{
    FTM0_C4V = plane_ticks[scan_plane];
    digitalWrite(pinADD_A, (prev_scan_row) & 1);
    digitalWrite(pinADD_B, (prev_scan_row) & 2);
    digitalWrite(pinADD_C, (prev_scan_row) & 4);
    digitalWrite(pinADD_D, (prev_scan_row) & 8);
    digitalWrite(pinLAT, 1);
    digitalWrite(pinLAT, 0);


    //do this here to allow time for vid/screen base to be swapped after an L/V sync
    register unsigned int *sp_0 = (unsigned int *)vid_base + SCR_WIDTH * scan_row;      
    register unsigned int *sp_2 = sp_0 + SCR_WIDTH * MAX_ROWS * 2;             

    digitalWrite(17, 1);
    register int x;
    register int z;
    register int sm_R = scan_mask_R;
    register int sm_G = scan_mask_G;
    register int sm_B = scan_mask_B;
    
    for (register int i = 0; i < SCR_WIDTH; i++) {

      PP_PORT_L_C = (PB_R | PB_G | PB_B | PB_R1 | PB_G1 | PB_B1);
      PP_PORT_H_C = (PB_R2 | PB_G2 | PB_B2 | PB_R3 | PB_G3 | PB_B3);

      z = 0;      
      x = sp_0[0];
      DWR(PB_R, sm_R);
      DWR(PB_G, sm_G);
      DWR(PB_B, sm_B);
      //sp_0++;


      x = sp_0[SCR_WIDTH * MAX_ROWS];
      DWR(PB_R1, sm_R);
      DWR(PB_G1, sm_G);
      DWR(PB_B1, sm_B);
      sp_0++;

      PP_PORT_L_S = z;
      PP_CLK_C = PB_CLK;

      z = 0;      
      x = sp_2[i];
      DWR(PB_R2, sm_R);
      DWR(PB_G2, sm_G);
      DWR(PB_B2, sm_B);

      x = sp_2[i + SCR_WIDTH * MAX_ROWS];
      DWR(PB_R3, sm_R);
      DWR(PB_G3, sm_G);
      DWR(PB_B3, sm_B);

      PP_PORT_H_S = z;



      
      PP_CLK_S = PB_CLK;
    }

    digitalWrite(17, 0);

    prev_scan_plane = scan_plane;
    prev_scan_row = scan_row;
    scan_plane++;
    if (scan_plane >= MAX_PLANES) 
    {
      scan_plane = 0;
      scan_mask_R = PLANE_MASK_MAX_R;
      scan_mask_G = PLANE_MASK_MAX_G;
      scan_mask_B = PLANE_MASK_MAX_B;
      scan_row++;
      if (scan_row >= MAX_ROWS)
      {
        scan_row = 0;
        frame++;
      }
    } else {
      scan_mask_R = scan_mask_R >> 1;
      scan_mask_G = scan_mask_G >> 1;
      scan_mask_B = scan_mask_B >> 1;

    }


    

}

void DomRGBScreen::waitVSync() {
  int f = frame;
  while (f == frame) ;

}

void DomRGBScreen::waitLSync() {
  unsigned char f = scan_row;
  while (f == scan_row) ;

}


DomRGBScreen _domRGBScreen(0);
